﻿using UnityEngine;
using System.Collections;

public class Projectile : MonoBehaviour
{
	// 画像編集用ソフトのようにレイヤーを用意して、
	// すべてのオブジェクトに対して衝突判定をせず、そのレイヤーの
	// オブジェクトに対して衝突判定を行い、パフォーマンスを向上させる。
	public LayerMask collisionMask;

	public Color trailColour;

	float speed = 10;
	float damage = 1;
	float skinWidth = .1f;

	float lifetime = 3;

	void Start ()
	{
		// TODO: Object Poolingができるのはず。
		Destroy (gameObject, lifetime);

		// 打った瞬間に敵に、銃口が埋まっている場合にColliderが取得できる。
		Collider[] initialCollisions = Physics.OverlapSphere (transform.position, .1f, collisionMask);
		if (initialCollisions.Length > 0) {
			OnHitObject (initialCollisions [0], transform.position);
		}

		GetComponent<TrailRenderer> ().material.SetColor ("_TintColor", trailColour);
	}

	public void SetSpeed (float newSpeed)
	{
		speed = newSpeed;
	}

	void Update ()
	{
		float moveDistance = speed * Time.deltaTime;
		CheckCollisions (moveDistance);
		transform.Translate (Vector3.forward * moveDistance);
	}

	void CheckCollisions (float moveDistance)
	{
		// Projectileの場所から、向いている方向への光線。
		Ray ray = new Ray (transform.position, transform.forward);
		RaycastHit hit;

		// 最後の引数で、常にトリガーヒット(IsTrigger)を報告する。
		if (Physics.Raycast (ray, out hit, moveDistance + skinWidth, collisionMask, QueryTriggerInteraction.Collide)) {
			Debug.DrawLine (ray.origin, hit.point, Color.red);
			OnHitObject (hit.collider, hit.point);
		}
	}

	// 銃口が敵に埋まっている時
	void OnHitObject (Collider c, Vector3 hitPoint)
	{
		// プロジェクタイルが当たったIDamageable対象オブジェクト。
		IDamageable damageableObject = c.gameObject.GetComponent<IDamageable> ();
		if (damageableObject != null) {
			damageableObject.TakeHit (damage, hitPoint, transform.forward);
		}
		GameObject.Destroy (gameObject);
	}
}
